@@ -24,7 +24,7 @@ import xmlrpclib, random, errno, string
 
 from moosic.utilities import *
 from moosic.client.factory import startServer
-from moosic.client.cli import VERSION
+from moosic import VERSION
 
 # Define the True and False constants if they don't already exist.
 try: True
@@ -136,7 +136,12 @@ def check_args(command, arglist):
     # before processing the rest of their argument lists.
     if command == 'insert' or command == 'pl-insert':
         try:
-            dispatcher['insert'].position = int(arglist.pop())
+            index = arglist.pop()
+            if not (index[0].isdigit() or index[0] == '-'):
+                index = index[1:]
+            if not (index[-1].isdigit() or index[-1] == '-'):
+                index = index[:-1]
+            dispatcher['insert'].position = int(index)
         except ValueError, e:
             print >>err, "Error:", e
             print >>err, "An integer is required as the final argument."
@@ -262,20 +267,25 @@ def process_filelist(moosic, arglist, opts):
         arglist.sort()
     if opts['no-unplayables']:
         playable_patterns = [entry[0].data for entry in moosic.getconfig()]
-        arglist = [i for i in arglist for pat in playable_patterns
-                        if re.search(pat, i)]
+        newlist = []
+        for i in arglist:
+            for pat in playable_patterns:
+                if re.search(pat, i):
+                    newlist.append(i)
+                    break
+        arglist = newlist
     return arglist
 
 #---------------------------- Dispatcher functions ----------------------------#
 
 def start_server(moosic, arglist, opts):
-    "start-server [options] - Start the server when it isn't already running."
+    "start-server [server-options] - Start the server when it isn't already running."
     if opts['tcp-address']:
         print >>sys.stderr, \
                 wrap("Warning: the Moosic server is being started on the local "
                      "computer, even though you specified the -t option, which "
                      "usually causes this client to interact with a server on "
-                     "a remote computer.")
+                     "a remote computer. This is probably not what you want.")
     if not arglist:
         arglist = ['-c', opts['config-dir']]
     return startServer(*(['moosicd'] + arglist))
@@ -587,7 +597,12 @@ def move(moosic, arglist, opts):
     '''move <range> <index> - Moves all items in the given range to a new position in
     the song queue.'''
     start, end = parse_range(arglist[0], 0, None)
-    dest = int(arglist[1])
+    dest = arglist[1]
+    if not (dest[0].isdigit() or dest[0] == '-'):
+        dest = dest[1:]
+    if not (dest[-1].isdigit() or dest[-1] == '-'):
+        dest = dest[:-1]
+    dest = int(dest)
     if end is None:
         moosic.move((start,), dest)
     else:
@@ -721,7 +736,7 @@ def stagger(moosic, arglist, opts):
         items = antigrep(pattern, items)
     # Perform a staggered merge on all the lists we collected, along
     # with any remaining items.
-    items = staggered_merge(*(list_of_lists)) + tuple(items)
+    items = staggered_merge(*(list_of_lists)) + items
     items = [xmlrpclib.Binary(i) for i in items]
     moosic.replace(items)
 
@@ -871,8 +886,7 @@ def list_(moosic, arglist, opts):
             index += 1
         except IOError, e:
             if e[0] == errno.EPIPE:
-                # Ignore "broken pipe" errors.
-                break
+                break # Ignore "broken pipe" errors.
             else:
                 raise e
 
@@ -1114,6 +1128,50 @@ dispatcher[unmangle(f.__name__)] = f
 
 #----------------------------#
 
+def goto(moosic, arglist, opts):
+    'goto <regex> - Jump to the first song in the queue that matches the regex.'
+    queue = moosic.list()
+    dest = 1
+    while queue:
+        if re.search(arglist[0], queue[0].data):
+            break
+        dest += 1
+        queue.pop(0)
+    if queue:
+        if not moosic.current().data:
+            dest -= 1
+        moosic.next(dest)
+    else:
+        print 'No match found:', arglist[0]
+
+f = goto
+f.category = 'manage'
+f.num_args = 'one'
+dispatcher[unmangle(f.__name__)] = f
+
+#----------------------------#
+
+def gobackto(moosic, arglist, opts):
+    'gobackto <regex> - Jump to the previous song that matches the regex.'
+    history = moosic.history()
+    dest = 1
+    while history:
+        if re.search(arglist[0], history[-1][0].data):
+            break
+        dest += 1
+        history.pop()
+    if history:
+        moosic.previous(dest)
+    else:
+        print 'No match found:', arglist[0]
+
+f = gobackto
+f.category = 'manage'
+f.num_args = 'one'
+dispatcher[unmangle(f.__name__)] = f
+
+#----------------------------#
+
 def noadvance(moosic, arglist, opts):
     'no-advance - Stop playing any new songs, but without interrupting the current song.'
     moosic.halt_queue()
