@@ -19,6 +19,7 @@
 import sys, os, os.path, string, time, threading, re
 import socket, SocketServer, signal, atexit
 import cPickle as pickle
+from moosic import VERSION
 from moosic.server.daemonize import daemonize
 
 # Define the True and False constants if they don't already exist.
@@ -138,7 +139,10 @@ def play(config, songname):
             data.log(Log.ERROR,
               'Cannot open player log file "%s": %s' % (e.songname, e.strerror))
         # Execute the command.
-        os.execvp(command[0], command)
+        try:
+            os.execvp(command[0], command)
+        except StandardError, e:
+            print 'Could not execute "%s": %s' % (' '.join(command), e)
     else:
         os.waitpid(data.player_pid, 0)
         data.player_pid = None
@@ -151,6 +155,7 @@ def queue_consumer():
     This procedure is a simple indefinite loop which is responsible for
     consuming the items in moosicd's queue.
     """
+    from moosic.server.methods import current_time
     while not data.quitFlag:
         if data.song_queue and data.qrunning:
             data.lock.acquire()
@@ -161,7 +166,6 @@ def queue_consumer():
                 data.last_queue_update = time.time()
                 data.song_start_event = time.time()
                 data.current_paused_time = 0
-                #data.start_stop_times = [-time.time()]  # old algorithm
             finally:
                 data.lock.release()
 
@@ -170,7 +174,9 @@ def queue_consumer():
             play(data.config, data.current_song)
             # Note that control does not return to this point until the
             # song is finished playing.
-            data.log(Log.NOTICE, 'Finished playing ' + data.current_song)
+            data.log(Log.NOTICE, 'Finished playing ' + data.current_song +
+                    ' (total playing time: %s)' %
+                    time.strftime('%H:%M:%S', time.gmtime(current_time())))
 
             data.lock.acquire()
             try:
@@ -212,7 +218,7 @@ def handleOptions(argv, defaultOpts):
         sys.exit('Option processing error: %s' % e)
     for opt, val in options:
         if opt == '-h' or opt == '--help':
-            print 'usage:', os.path.basename(sys.argv[0]), '[options]' + '''
+            print 'usage:', os.path.basename(argv[0]), '[options]' + '''
     Options:
         -s, --history-size <num> Sets the maximum size of the history list.
                                  (Default: 50)
@@ -244,7 +250,7 @@ def handleOptions(argv, defaultOpts):
         -h, --help          Print this help text and exit.'''
             sys.exit(0)
         if opt == '-v' or opt == '--version':
-            print "moosicd", moosic.server.VERSION
+            print "moosicd", VERSION
             print """
 Copyright (C) 2001-2003 Daniel Pearson <daniel@nanoo.org>
 This is free software; see the source for copying conditions.  There is NO
@@ -336,9 +342,12 @@ def main(argv):
         except IOError, e:
             data.log(Log.WARNING,
               'Cannot open saved-state file "%s": %s' % (e.filename,e.strerror))
-        except pickle.PicklingError, e:
+        except pickle.PickleError, e:
             data.log(Log.WARNING,
               'Unpickling error: %s\nCannot load saved state.' % (e))
+        except:
+            data.log(Log.WARNING,
+              'Saved-state file "%s" could not be loaded.' % (savefilename))
 
     # Allow the max_hist_size specified in the command-line options to override
     # the value from the saved state.
@@ -434,12 +443,15 @@ def main(argv):
         except PicklingError, e:
             data.log(Log.WARNING,
               'Pickling error: %s\nCannot save state.' % (e))
-    def save_timer():
-        savestate()
-        t = threading.Timer(300, save_timer)
+
+    def save_timer(prev_update):
+        if data.last_queue_update != prev_update: # Don't bother waking up the
+            savestate()                           # hard disk if nothing has
+                                                  # changed.
+        t = threading.Timer(300, save_timer, args=(data.last_queue_update,))
         t.setDaemon(True)
         t.start()
-    save_timer()
+    save_timer(0)
 
     # Set up the signal handlers and exit handler.
     def reconfig(signum=None, stackframe=None):
@@ -448,11 +460,12 @@ def main(argv):
     signal.signal(signal.SIGHUP, reconfig)
 
     def quit(signum=None, stackframe=None):
-       #if stackframe:                        # DEBUG
-       #    import traceback                  #
-       #    traceback.print_stack(stackframe) #
+       #if stackframe:                                              # DEBUG
+       #    import traceback                                        #
+       #    data.log(Log.DEBUG, traceback.format_stack(stackframe)) #
         if signum:
-            data.log(Log.NOTICE, "Killed by signal %d." % signum)
+            data.log(Log.NOTICE, "Killed by signal %d (PID: %d)." %
+                                 (signum, os.getpid))
         sys.exit()
     signal.signal(signal.SIGINT, quit)
     signal.signal(signal.SIGTERM, quit)
